%% Example_2.m
%
% Simulation of a sequence of saccade waveforms using a parametric model.
%
% The saccade model corresponds to the 'main sequence' formula
%    Vp = eta*(1 - exp(-A/c))
% where Vp is the peak saccadic velocity and A is the saccadic amplitude.
%
% In this simulation, the main sequence parameters 'eta' and 'c' 
% and the amplitude of each saccade are chosen randomly in a given range.
% The intervals between the saccades are also randomized.

eta_min = 500;                      % main sequence parameter (degree/sec)
eta_max = 590;

c_min = 4.5;                        % main sequence parameter (no units)
c_max = 7.5;

Fs = 500;                           % sampling rate (samples/sec)
N = 50;                             % number of saccades
all_saccades = zeros(1, Fs);        % start with one second of no saccades
Amp = zeros(1, N);                  % amplitude of saccades (degree)
Vp = zeros(1, N);                   % peak velocity of saccades (degree/sec)
direction = 1;                      % direction of saccades
for i = 1:N
    t_min = -0.2;
    t_max = round(rand*1000)/1000 + 0.2;
    t = t_min:1/Fs:t_max;                       % (sec)
    eta = eta_min + (eta_max - eta_min)*rand;   % (degree/sec)
    c = c_min + (c_max - c_min)*rand;           % (no units)
    Amp(i) = 1 + 25*rand;                       % amplitude (degree)
    Vp(i) = eta * (1 - exp(-Amp(i)/c));         % peak velocity (degree/sec)
    [waveform, velocity] = saccade_model(t, eta, c, Amp(i));

    % update direction of saccade to keep angle between -30 and +30
    waveform = direction * waveform;
    if abs(all_saccades(end) + waveform(end)) > 29
        waveform = -waveform;
        direction = -direction;
    end
    
    % concatenate saccades
    all_saccades = [all_saccades, waveform + all_saccades(end)];
end
all_saccades = [all_saccades, ones(1,Fs)*all_saccades(end)]; % one sec. no saccades
L = length(all_saccades);
t = (0:L-1)/Fs;                      % total time (sec)

%% Plots

% Main sequence plot
figure(1)
clf
subplot(2, 2, 1)
plot(Amp, Vp, 'o')
xlabel('Saccade amplitude (deg)')
ylabel('Saccade peak velocity (deg/s)')
title('Main sequence')

% Position trace plot
subplot(2, 1, 2)
plot(t, all_saccades)
xlim(t([1 end]))
xlabel('Time (s)')
ylabel('Angle (deg)')
title('Series of simulated saccades')

print -dpdf Example_2 

